# This file is automatically generated by botocraft.  Do not edit directly.
# pylint: disable=anomalous-backslash-in-string,unsubscriptable-object,line-too-long,arguments-differ,arguments-renamed
# mypy: disable-error-code="index, override, assignment"
from collections import OrderedDict
from datetime import datetime
from functools import cached_property
from typing import Any, ClassVar, Dict, List, Literal, Optional, Type, cast

from pydantic import Field

from botocraft.mixins.tags import TagsDictMixin
from botocraft.services.common import Tag
from botocraft.services.ec2 import (SecurityGroup, SecurityGroupManager, Vpc,
                                    VpcManager)

from .abstract import (Boto3Model, Boto3ModelManager, PrimaryBoto3Model,
                       ReadonlyBoto3Model, ReadonlyBoto3ModelManager,
                       ReadonlyPrimaryBoto3Model)

# ===============
# Managers
# ===============


class LoadBalancerManager(Boto3ModelManager):
    service_name: str = "elbv2"

    def create(
        self,
        model: "LoadBalancer",
        SubnetMappings: Optional[List["SubnetMapping"]] = None,
        Tags: Optional[List[Tag]] = None,
    ) -> "LoadBalancer":
        """
        Creates an Application Load Balancer, Network Load Balancer, or Gateway
        Load Balancer.

        Args:
            model: The :py:class:`LoadBalancer` to create.

        Keyword Args:
            SubnetMappings: The IDs of the public subnets. You can specify only one
                subnet per Availability Zone. You must specify either subnets or subnet
                mappings, but not both.
            Tags: The tags to assign to the load balancer.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Name=data["LoadBalancerName"],
            Subnets=data.get("Subnets"),
            SubnetMappings=self.serialize(SubnetMappings),
            SecurityGroups=data.get("SecurityGroups"),
            Scheme=data.get("Scheme"),
            Tags=self.serialize(Tags),
            Type=data.get("Type"),
            IpAddressType=data.get("IpAddressType"),
            CustomerOwnedIpv4Pool=data.get("CustomerOwnedIpv4Pool"),
        )
        _response = self.client.create_load_balancer(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateLoadBalancerOutput(**_response)

        return cast("LoadBalancer", response.LoadBalancers[0])

    def delete(self, LoadBalancerArn: str) -> None:
        """
        Deletes the specified Application Load Balancer, Network Load Balancer,
        or Gateway Load Balancer. Deleting a load balancer also deletes its
        listeners.

        Args:
            LoadBalancerArn: The Amazon Resource Name (ARN) of the load balancer.
        """
        args: Dict[str, Any] = dict(LoadBalancerArn=self.serialize(LoadBalancerArn))
        self.client.delete_load_balancer(
            **{k: v for k, v in args.items() if v is not None}
        )

    def get(
        self, *, LoadBalancerArn: Optional[str] = None, Name: Optional[str] = None
    ) -> Optional["LoadBalancer"]:
        """
        Describes the specified load balancers or all of your load balancers.

        Keyword Args:
            LoadBalancerArn: The Amazon Resource Names (ARN) of the load balancer.
            Name: The names of the load balancer.
        """
        args: Dict[str, Any] = dict(
            LoadBalancerArns=self.serialize([LoadBalancerArn]),
            Names=self.serialize([Name]),
        )
        _response = self.client.describe_load_balancers(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DescribeLoadBalancersOutput(**_response)

        if response.LoadBalancers:
            return response.LoadBalancers[0]
        return None

    def list(
        self,
        *,
        LoadBalancerArns: Optional[List[str]] = None,
        Names: Optional[List[str]] = None
    ) -> List["LoadBalancer"]:
        """
        Describes the specified load balancers or all of your load balancers.

        Keyword Args:
            LoadBalancerArns: The Amazon Resource Names (ARN) of the load balancers.
                You can specify up to 20 load balancers in a single call.
            Names: The names of the load balancers.
        """
        paginator = self.client.get_paginator("describe_load_balancers")
        args: Dict[str, Any] = dict(
            LoadBalancerArns=self.serialize(LoadBalancerArns),
            Names=self.serialize(Names),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results: List["LoadBalancer"] = []
        for _response in response_iterator:
            response = DescribeLoadBalancersOutput(**_response)
            if response.LoadBalancers:
                results.extend(response.LoadBalancers)
            else:
                break
        return results


class ListenerManager(Boto3ModelManager):
    service_name: str = "elbv2"

    def create(self, model: "Listener", Tags: Optional[List[Tag]] = None) -> "Listener":
        """
        Creates a listener for the specified Application Load Balancer, Network
        Load Balancer, or Gateway Load Balancer.

        Args:
            model: The :py:class:`Listener` to create.

        Keyword Args:
            Tags: The tags to assign to the listener.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            LoadBalancerArn=data.get("LoadBalancerArn"),
            DefaultActions=data.get("DefaultActions"),
            Protocol=data.get("Protocol"),
            Port=data.get("Port"),
            SslPolicy=data.get("SslPolicy"),
            Certificates=data.get("Certificates"),
            AlpnPolicy=data.get("AlpnPolicy"),
            Tags=self.serialize(Tags),
        )
        _response = self.client.create_listener(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateListenerOutput(**_response)

        return cast("Listener", response.Listeners[0])

    def update(self, model: "Listener") -> "Listener":
        """
        Replaces the specified properties of the specified listener. Any
        properties that you do not specify remain unchanged.

        Args:
            model: The :py:class:`Listener` to update.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            ListenerArn=data.get("ListenerArn"),
            Port=data.get("Port"),
            Protocol=data.get("Protocol"),
            SslPolicy=data.get("SslPolicy"),
            Certificates=data.get("Certificates"),
            DefaultActions=data.get("DefaultActions"),
            AlpnPolicy=data.get("AlpnPolicy"),
        )
        _response = self.client.modify_listener(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ModifyListenerOutput(**_response)

        return cast("Listener", response.Listeners[0])

    def delete(self, ListenerArn: str) -> None:
        """
        Deletes the specified listener.

        Args:
            ListenerArn: The Amazon Resource Name (ARN) of the listener.
        """
        args: Dict[str, Any] = dict(ListenerArn=self.serialize(ListenerArn))
        self.client.delete_listener(**{k: v for k, v in args.items() if v is not None})

    def get(self, ListenerArn: str) -> Optional["Listener"]:
        """
        Describes the specified listeners or the listeners for the specified
        Application Load Balancer, Network Load Balancer, or Gateway Load
        Balancer. You must specify either a load balancer or one or more
        listeners.

        Args:
            ListenerArn: The Amazon Resource Names (ARN) of the listener.
        """
        args: Dict[str, Any] = dict(ListenerArns=self.serialize([ListenerArn]))
        _response = self.client.describe_listeners(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DescribeListenersOutput(**_response)

        if response.Listeners:
            return response.Listeners[0]
        return None

    def list(
        self,
        *,
        LoadBalancerArn: Optional[str] = None,
        ListenerArns: Optional[List[str]] = None
    ) -> List["Listener"]:
        """
        Describes the specified listeners or the listeners for the specified
        Application Load Balancer, Network Load Balancer, or Gateway Load
        Balancer. You must specify either a load balancer or one or more
        listeners.

        Keyword Args:
            LoadBalancerArn: The Amazon Resource Name (ARN) of the load balancer.
            ListenerArns: The Amazon Resource Names (ARN) of the listeners.
        """
        paginator = self.client.get_paginator("describe_listeners")
        args: Dict[str, Any] = dict(
            LoadBalancerArn=self.serialize(LoadBalancerArn),
            ListenerArns=self.serialize(ListenerArns),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results: List["Listener"] = []
        for _response in response_iterator:
            response = DescribeListenersOutput(**_response)
            if response.Listeners:
                results.extend(response.Listeners)
            else:
                break
        return results


class RuleManager(Boto3ModelManager):
    service_name: str = "elbv2"

    def create(
        self, model: "Rule", ListenerArn: str, Tags: Optional[List[Tag]] = None
    ) -> "Rule":
        """
        Creates a rule for the specified listener. The listener must be
        associated with an Application Load Balancer.

        Args:
            model: The :py:class:`Rule` to create.
            ListenerArn: The Amazon Resource Name (ARN) of the listener.

        Keyword Args:
            Tags: The tags to assign to the rule.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            ListenerArn=self.serialize(ListenerArn),
            Conditions=data.get("Conditions"),
            Priority=data.get("Priority"),
            Actions=data.get("Actions"),
            Tags=self.serialize(Tags),
        )
        _response = self.client.create_rule(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateRuleOutput(**_response)

        return cast("Rule", response.Rules[0])

    def update(self, model: "Rule") -> "Rule":
        """
        Replaces the specified properties of the specified rule. Any properties
        that you do not specify are unchanged.

        Args:
            model: The :py:class:`Rule` to update.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            RuleArn=data.get("RuleArn"),
            Conditions=data.get("Conditions"),
            Actions=data.get("Actions"),
        )
        _response = self.client.modify_rule(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ModifyRuleOutput(**_response)

        return cast("Rule", response.Rules[0])

    def delete(self, RuleArn: str) -> None:
        """
        Deletes the specified rule.

        Args:
            RuleArn: The Amazon Resource Name (ARN) of the rule.
        """
        args: Dict[str, Any] = dict(RuleArn=self.serialize(RuleArn))
        self.client.delete_rule(**{k: v for k, v in args.items() if v is not None})

    def get(self, RuleArn: str) -> Optional["Rule"]:
        """
        Describes the specified rules or the rules for the specified listener.
        You must specify either a listener or one or more rules.

        Args:
            RuleArn: The Amazon Resource Names (ARN) of the rule.
        """
        args: Dict[str, Any] = dict(RuleArns=self.serialize([RuleArn]))
        _response = self.client.describe_rules(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DescribeRulesOutput(**_response)

        if response.Rules:
            return response.Rules[0]
        return None

    def list(
        self, *, ListenerArn: Optional[str] = None, RuleArns: Optional[List[str]] = None
    ) -> List["Rule"]:
        """
        Describes the specified rules or the rules for the specified listener.
        You must specify either a listener or one or more rules.

        Keyword Args:
            ListenerArn: The Amazon Resource Name (ARN) of the listener.
            RuleArns: The Amazon Resource Names (ARN) of the rules.
        """
        paginator = self.client.get_paginator("describe_rules")
        args: Dict[str, Any] = dict(
            ListenerArn=self.serialize(ListenerArn), RuleArns=self.serialize(RuleArns)
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results: List["Rule"] = []
        for _response in response_iterator:
            response = DescribeRulesOutput(**_response)
            if response.Rules:
                results.extend(response.Rules)
            else:
                break
        return results


class TargetGroupManager(Boto3ModelManager):
    service_name: str = "elbv2"

    def create(
        self, model: "TargetGroup", Name: str, Tags: Optional[List[Tag]] = None
    ) -> "TargetGroup":
        """
        Creates a target group.

        Args:
            model: The :py:class:`TargetGroup` to create.
            Name: The name of the target group.

        Keyword Args:
            Tags: The tags to assign to the target group.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Name=self.serialize(Name),
            Protocol=data.get("Protocol"),
            ProtocolVersion=data.get("ProtocolVersion"),
            Port=data.get("Port"),
            VpcId=data.get("VpcId"),
            HealthCheckProtocol=data.get("HealthCheckProtocol"),
            HealthCheckPort=data.get("HealthCheckPort"),
            HealthCheckEnabled=data.get("HealthCheckEnabled"),
            HealthCheckPath=data.get("HealthCheckPath"),
            HealthCheckIntervalSeconds=data.get("HealthCheckIntervalSeconds"),
            HealthCheckTimeoutSeconds=data.get("HealthCheckTimeoutSeconds"),
            HealthyThresholdCount=data.get("HealthyThresholdCount"),
            UnhealthyThresholdCount=data.get("UnhealthyThresholdCount"),
            Matcher=data.get("Matcher"),
            TargetType=data.get("TargetType"),
            Tags=self.serialize(Tags),
            IpAddressType=data.get("IpAddressType"),
        )
        _response = self.client.create_target_group(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateTargetGroupOutput(**_response)

        return cast("TargetGroup", response.TargetGroups[0])

    def update(self, model: "TargetGroup") -> "TargetGroup":
        """
        Modifies the health checks used when evaluating the health state of the
        targets in the specified target group.

        Args:
            model: The :py:class:`TargetGroup` to update.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            TargetGroupArn=data.get("TargetGroupArn"),
            HealthCheckProtocol=data.get("HealthCheckProtocol"),
            HealthCheckPort=data.get("HealthCheckPort"),
            HealthCheckPath=data.get("HealthCheckPath"),
            HealthCheckEnabled=data.get("HealthCheckEnabled"),
            HealthCheckIntervalSeconds=data.get("HealthCheckIntervalSeconds"),
            HealthCheckTimeoutSeconds=data.get("HealthCheckTimeoutSeconds"),
            HealthyThresholdCount=data.get("HealthyThresholdCount"),
            UnhealthyThresholdCount=data.get("UnhealthyThresholdCount"),
            Matcher=data.get("Matcher"),
        )
        _response = self.client.modify_target_group(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ModifyTargetGroupOutput(**_response)

        return cast("TargetGroup", response.TargetGroups[0])

    def delete(self, TargetGroupArn: str) -> None:
        """
        Deletes the specified target group.

        Args:
            TargetGroupArn: The Amazon Resource Name (ARN) of the target group.
        """
        args: Dict[str, Any] = dict(TargetGroupArn=self.serialize(TargetGroupArn))
        self.client.delete_target_group(
            **{k: v for k, v in args.items() if v is not None}
        )

    def get(
        self, *, TargetGroupArn: Optional[str] = None, Name: Optional[str] = None
    ) -> Optional["TargetGroup"]:
        """
        Describes the specified target groups or all of your target groups. By default,
        all target groups are described. Alternatively, you can specify one of the
        following to filter the results: the ARN of the load balancer, the names of one
        or more target groups, or the ARNs of one or more target groups.

        Keyword Args:
            TargetGroupArn: The Amazon Resource Names (ARN) of the target group.
            Name: The name of the target group.

        """
        args: Dict[str, Any] = dict(
            TargetGroupArns=self.serialize([TargetGroupArn]),
            Names=self.serialize([Name]),
        )
        _response = self.client.describe_target_groups(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DescribeTargetGroupsOutput(**_response)

        if response.TargetGroups:
            return response.TargetGroups[0]
        return None

    def list(
        self,
        *,
        LoadBalancerArn: Optional[str] = None,
        TargetGroupArns: Optional[List[str]] = None,
        Names: Optional[List[str]] = None
    ) -> List["TargetGroup"]:
        """
        Describes the specified target groups or all of your target groups. By default,
        all target groups are described. Alternatively, you can specify one of the
        following to filter the results: the ARN of the load balancer, the names of one
        or more target groups, or the ARNs of one or more target groups.

        Keyword Args:
            LoadBalancerArn: The Amazon Resource Name (ARN) of the load balancer.
            TargetGroupArns: The Amazon Resource Names (ARN) of the target groups.
            Names: The names of the target groups.

        """
        paginator = self.client.get_paginator("describe_target_groups")
        args: Dict[str, Any] = dict(
            LoadBalancerArn=self.serialize(LoadBalancerArn),
            TargetGroupArns=self.serialize(TargetGroupArns),
            Names=self.serialize(Names),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results: List["TargetGroup"] = []
        for _response in response_iterator:
            response = DescribeTargetGroupsOutput(**_response)
            if response.TargetGroups:
                results.extend(response.TargetGroups)
            else:
                break
        return results


# ==============
# Service Models
# ==============


class LoadBalancerState(Boto3Model):
    """
    The state of the load balancer.
    """

    #: The state code. The initial state of the load balancer is ``provisioning``.
    #: After the load balancer is fully set up and ready to route traffic, its state
    #: is ``active``. If load balancer is routing traffic but does not have the
    #: resources it needs to scale, its state is``active_impaired``. If the load
    #: balancer could not be set up, its state is ``failed``.
    Code: Optional[
        Literal["active", "provisioning", "active_impaired", "failed"]
    ] = None
    #: A description of the state.
    Reason: Optional[str] = None


class LoadBalancerAddress(Boto3Model):
    """
    Information about a static IP address for a load balancer.
    """

    #: The static IP address.
    IpAddress: Optional[str] = None
    #: [Network Load Balancers] The allocation ID of the Elastic IP address for an
    #: internal-facing load balancer.
    AllocationId: Optional[str] = None
    #: [Network Load Balancers] The private IPv4 address for an internal load
    #: balancer.
    PrivateIPv4Address: Optional[str] = None
    #: [Network Load Balancers] The IPv6 address.
    IPv6Address: Optional[str] = None


class AvailabilityZone(Boto3Model):
    """
    Information about an Availability Zone.
    """

    #: The name of the Availability Zone.
    ZoneName: Optional[str] = None
    #: The ID of the subnet. You can specify one subnet per Availability Zone.
    SubnetId: Optional[str] = None
    #: [Application Load Balancers on Outposts] The ID of the Outpost.
    OutpostId: Optional[str] = None
    #: [Network Load Balancers] If you need static IP addresses for your load
    #: balancer, you can specify one Elastic IP address per Availability Zone when you
    #: create an internal-facing load balancer. For internal load balancers, you can
    #: specify a private IP address from the IPv4 range of the subnet.
    LoadBalancerAddresses: Optional[List["LoadBalancerAddress"]] = None


class LoadBalancer(PrimaryBoto3Model):
    """
    Information about a load balancer.
    """

    objects: ClassVar[Boto3ModelManager] = LoadBalancerManager()

    #: The name of the load balancer.
    LoadBalancerName: str
    #: The nodes of an Internet-facing load balancer have public IP addresses. The DNS
    #: name of an Internet-facing load balancer is publicly resolvable to the public
    #: IP addresses of the nodes. Therefore, Internet-facing load balancers can route
    #: requests from clients over the internet.
    Scheme: Optional[Literal["internet-facing", "internal"]] = "internet-facing"
    #: The type of load balancer.
    Type: Optional[Literal["application", "network", "gateway"]] = "application"
    #: The type of IP addresses used by the subnets for your load balancer. The
    #: possible values are ``ipv4`` (for IPv4 addresses) and ``dualstack`` (for IPv4
    #: and IPv6 addresses).
    IpAddressType: Optional[Literal["ipv4", "dualstack"]] = "ipv4"
    #: The Amazon Resource Name (ARN) of the load balancer.
    LoadBalancerArn: str = Field(default=None, frozen=True)
    #: The public DNS name of the load balancer.
    DNSName: str = Field(default=None, frozen=True)
    #: The ID of the Amazon Route 53 hosted zone associated with the load balancer.
    CanonicalHostedZoneId: str = Field(default=None, frozen=True)
    #: The date and time the load balancer was created.
    CreatedTime: datetime = Field(default=None, frozen=True)
    #: The ID of the VPC for the load balancer.
    VpcId: str = Field(default=None, frozen=True)
    #: The state of the load balancer.
    State: LoadBalancerState = Field(default=None, frozen=True)
    #: The subnets for the load balancer.
    AvailabilityZones: List["AvailabilityZone"] = Field(default=None, frozen=True)
    #: The IDs of the security groups for the load balancer.
    SecurityGroups: Optional[List[str]] = None
    #: [Application Load Balancers on Outposts] The ID of the customer-owned address
    #: pool.
    CustomerOwnedIpv4Pool: Optional[str] = None
    #: Indicates whether to evaluate inbound security group rules for traffic sent to
    #: a Network Load Balancer through Amazon Web Services PrivateLink.
    EnforceSecurityGroupInboundRulesOnPrivateLinkTraffic: str = Field(
        default=None, frozen=True
    )

    @property
    def pk(self) -> Optional[str]:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`LoadBalancerArn` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.LoadBalancerArn

    @property
    def arn(self) -> Optional[str]:
        """
        Return the ARN of the model.   This is the value of the
        :py:attr:`LoadBalancerArn` attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.LoadBalancerArn

    @property
    def name(self) -> Optional[str]:
        """
        Return the name of the model.   This is the value of the
        :py:attr:`LoadBalancerName` attribute.

        Returns:
            The name of the model instance.
        """
        return self.LoadBalancerName

    @cached_property
    def listeners(self) -> Optional[List["Listener"]]:
        """
        Return the ARNs of :py:class:`Listener` objects that belong to this
        load balancer, if any.
        """

        try:
            pk = OrderedDict(
                {
                    "load_balancer": self.LoadBalancerArn,
                }
            )
        except AttributeError:
            return []
        return Listener.objects.list(**pk)

    @cached_property
    def vpc(self) -> Optional["Vpc"]:
        """
        Return the :py:class:`Vpc` object that this load balancer belongs to,
        if any.
        """

        try:
            pk = OrderedDict(
                {
                    "VpcId": self.VpcId,
                }
            )
        except AttributeError:
            return None
        return Vpc.objects.get(**pk)

    @cached_property
    def security_groups(self) -> Optional[List["SecurityGroup"]]:
        """
        Return the ARNs of :py:class:`SecurityGroup` objects that belong to
        this load balancer, if any.
        """

        try:
            pk = OrderedDict(
                {
                    "GroupIds": self.SecurityGroups,
                }
            )
        except AttributeError:
            return []
        return SecurityGroup.objects.list(**pk)


class Certificate(ReadonlyBoto3Model):
    """
    Information about an SSL server certificate.
    """

    #: The Amazon Resource Name (ARN) of the certificate.
    CertificateArn: Optional[str] = None
    #: Indicates whether the certificate is the default certificate. Do not set this
    #: value when specifying a certificate as an input. This value is not included in
    #: the output when describing a listener, but is included when describing listener
    #: certificates.
    IsDefault: Optional[bool] = None


class AuthenticateOidcActionConfig(Boto3Model):
    """
    [HTTPS listeners] Information about an identity provider that is compliant
    with OpenID Connect (OIDC).

    Specify only when ``Type`` is ``authenticate-oidc``.
    """

    #: The OIDC issuer identifier of the IdP. This must be a full URL, including the
    #: HTTPS protocol, the domain, and the path.
    Issuer: str
    #: The authorization endpoint of the IdP. This must be a full URL, including the
    #: HTTPS protocol, the domain, and the path.
    AuthorizationEndpoint: str
    #: The token endpoint of the IdP. This must be a full URL, including the HTTPS
    #: protocol, the domain, and the path.
    TokenEndpoint: str
    #: The user info endpoint of the IdP. This must be a full URL, including the HTTPS
    #: protocol, the domain, and the path.
    UserInfoEndpoint: str
    #: The OAuth 2.0 client identifier.
    ClientId: str
    #: The OAuth 2.0 client secret. This parameter is required if you are creating a
    #: rule. If you are modifying a rule, you can omit this parameter if you set
    #: ``UseExistingClientSecret`` to true.
    ClientSecret: Optional[str] = None
    #: The name of the cookie used to maintain session information. The default is
    #: AWSELBAuthSessionCookie.
    SessionCookieName: Optional[str] = None
    #: The set of user claims to be requested from the IdP. The default is ``openid``.
    Scope: Optional[str] = None
    #: The maximum duration of the authentication session, in seconds. The default is
    #: 604800 seconds (7 days).
    SessionTimeout: Optional[int] = None
    #: The query parameters (up to 10) to include in the redirect request to the
    #: authorization endpoint.
    AuthenticationRequestExtraParams: Optional[Dict[str, str]] = None
    #: The behavior if the user is not authenticated. The following are possible
    #: values:
    OnUnauthenticatedRequest: Optional[Literal["deny", "allow", "authenticate"]] = None
    #: Indicates whether to use the existing client secret when modifying a rule. If
    #: you are creating a rule, you can omit this parameter or set it to false.
    UseExistingClientSecret: Optional[bool] = None


class AuthenticateCognitoActionConfig(Boto3Model):
    """
    [HTTPS listeners] Information for using Amazon Cognito to authenticate
    users.

    Specify only when ``Type`` is ``authenticate-cognito``.
    """

    #: The Amazon Resource Name (ARN) of the Amazon Cognito user pool.
    UserPoolArn: str
    #: The ID of the Amazon Cognito user pool client.
    UserPoolClientId: str
    #: The domain prefix or fully-qualified domain name of the Amazon Cognito user
    #: pool.
    UserPoolDomain: str
    #: The name of the cookie used to maintain session information. The default is
    #: AWSELBAuthSessionCookie.
    SessionCookieName: Optional[str] = None
    #: The set of user claims to be requested from the IdP. The default is ``openid``.
    Scope: Optional[str] = None
    #: The maximum duration of the authentication session, in seconds. The default is
    #: 604800 seconds (7 days).
    SessionTimeout: Optional[int] = None
    #: The query parameters (up to 10) to include in the redirect request to the
    #: authorization endpoint.
    AuthenticationRequestExtraParams: Optional[Dict[str, str]] = None
    #: The behavior if the user is not authenticated. The following are possible
    #: values:
    OnUnauthenticatedRequest: Optional[Literal["deny", "allow", "authenticate"]] = None


class RedirectActionConfig(Boto3Model):
    """
    [Application Load Balancer] Information for creating a redirect action.

    Specify
    only when ``Type`` is ``redirect``.
    """

    #: The protocol. You can specify HTTP, HTTPS, or #{protocol}. You can redirect
    #: HTTP to HTTP, HTTP to HTTPS, and HTTPS to HTTPS. You cannot redirect HTTPS to
    #: HTTP.
    Protocol: Optional[str] = None
    #: The port. You can specify a value from 1 to 65535 or #{port}.
    Port: Optional[str] = None
    #: The hostname. This component is not percent-encoded. The hostname can contain
    #: #{host}.
    Host: Optional[str] = None
    #: The absolute path, starting with the leading "/". This component is not
    #: percent-encoded. The path can contain #{host}, #{path}, and #{port}.
    Path: Optional[str] = None
    #: The query parameters, URL-encoded when necessary, but not percent-encoded. Do
    #: not include the leading "?", as it is automatically added. You can specify any
    #: of the reserved keywords.
    Query: Optional[str] = None
    #: The HTTP redirect code. The redirect is either permanent (HTTP 301) or
    #: temporary (HTTP 302).
    StatusCode: Literal["HTTP_301", "HTTP_302"]


class FixedResponseActionConfig(Boto3Model):
    """
    [Application Load Balancer] Information for creating an action that returns
    a custom HTTP response.

    Specify only when ``Type`` is ``fixed-response``.
    """

    #: The message.
    MessageBody: Optional[str] = None
    #: The HTTP response code (2XX, 4XX, or 5XX).
    StatusCode: str
    #: The content type.
    ContentType: Optional[str] = None


class TargetGroupTuple(Boto3Model):
    """
    Information about how traffic will be distributed between multiple target
    groups in a forward rule.
    """

    #: The Amazon Resource Name (ARN) of the target group.
    TargetGroupArn: Optional[str] = None
    #: The weight. The range is 0 to 999.
    Weight: Optional[int] = None


class ElbV2TargetGroupStickinessConfig(Boto3Model):
    """
    The target group stickiness for the rule.
    """

    #: Indicates whether target group stickiness is enabled.
    Enabled: Optional[bool] = None
    #: The time period, in seconds, during which requests from a client should be
    #: routed to the same target group. The range is 1-604800 seconds (7 days).
    DurationSeconds: Optional[int] = None


class ForwardActionConfig(Boto3Model):
    """
    Information for creating an action that distributes requests among one or
    more target groups.

    For Network Load Balancers, you can specify a single target
    group. Specify only when ``Type`` is ``forward``. If you specify both
    ``ForwardConfig`` and ``TargetGroupArn``, you can specify only one target group
    using ``ForwardConfig`` and it must be the same target group specified in
    ``TargetGroupArn``.
    """

    #: The target groups. For Network Load Balancers, you can specify a single target
    #: group.
    TargetGroups: Optional[List["TargetGroupTuple"]] = None
    #: The target group stickiness for the rule.
    TargetGroupStickinessConfig: Optional[ElbV2TargetGroupStickinessConfig] = None


class Action(Boto3Model):
    """
    Information about an action.

    Each rule must include exactly one of the following types of actions:
    ``forward``, ``fixed-response``, or ``redirect``, and it must be the last
    action to be performed.
    """

    #: The type of action.
    Type: Literal[
        "forward",
        "authenticate-oidc",
        "authenticate-cognito",
        "redirect",
        "fixed-response",
    ]
    #: The Amazon Resource Name (ARN) of the target group. Specify only when ``Type``
    #: is ``forward`` and you want to route to a single target group. To route to one
    #: or more target groups, use ``ForwardConfig`` instead.
    TargetGroupArn: Optional[str] = None
    #: [HTTPS listeners] Information about an identity provider that is compliant with
    #: OpenID Connect (OIDC). Specify only when ``Type`` is ``authenticate-oidc``.
    AuthenticateOidcConfig: Optional[AuthenticateOidcActionConfig] = None
    #: [HTTPS listeners] Information for using Amazon Cognito to authenticate users.
    #: Specify only when ``Type`` is ``authenticate-cognito``.
    AuthenticateCognitoConfig: Optional[AuthenticateCognitoActionConfig] = None
    #: The order for the action. This value is required for rules with multiple
    #: actions. The action with the lowest value for order is performed first.
    Order: Optional[int] = None
    #: [Application Load Balancer] Information for creating a redirect action. Specify
    #: only when ``Type`` is ``redirect``.
    RedirectConfig: Optional[RedirectActionConfig] = None
    #: [Application Load Balancer] Information for creating an action that returns a
    #: custom HTTP response. Specify only when ``Type`` is ``fixed-response``.
    FixedResponseConfig: Optional[FixedResponseActionConfig] = None
    #: Information for creating an action that distributes requests among one or more
    #: target groups. For Network Load Balancers, you can specify a single target
    #: group. Specify only when ``Type`` is ``forward``. If you specify both
    #: ``ForwardConfig`` and ``TargetGroupArn``, you can specify only one target group
    #: using ``ForwardConfig`` and it must be the same target group specified in
    #: ``TargetGroupArn``.
    ForwardConfig: Optional[ForwardActionConfig] = None


class Listener(PrimaryBoto3Model):
    """
    Information about a listener.
    """

    objects: ClassVar[Boto3ModelManager] = ListenerManager()

    #: The Amazon Resource Name (ARN) of the load balancer.
    LoadBalancerArn: str
    #: The port on which the load balancer is listening.
    Port: int
    #: The protocol for connections from clients to the load balancer.
    Protocol: Optional[
        Literal["HTTP", "HTTPS", "TCP", "TLS", "UDP", "TCP_UDP", "GENEVE"]
    ] = "HTTPS"
    #: The Amazon Resource Name (ARN) of the listener.
    ListenerArn: Optional[str] = None
    #: [HTTPS or TLS listener] The default certificate for the listener.
    Certificates: Optional[List["Certificate"]] = None
    #: [HTTPS or TLS listener] The security policy that defines which protocols and
    #: ciphers are supported.
    SslPolicy: Optional[str] = None
    #: The default actions for the listener.
    DefaultActions: Optional[List["Action"]] = None
    #: [TLS listener] The name of the Application-Layer Protocol Negotiation (ALPN)
    #: policy.
    AlpnPolicy: Optional[List[str]] = None

    @property
    def pk(self) -> Optional[str]:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`ListenerArn` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.ListenerArn

    @property
    def arn(self) -> Optional[str]:
        """
        Return the ARN of the model.   This is the value of the
        :py:attr:`ListenerArn` attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.ListenerArn

    @cached_property
    def load_balancer(self) -> Optional["LoadBalancer"]:
        """
        Return the :py:class:`LoadBalancer` object that this listener belongs
        to, if any.
        """

        try:
            pk = OrderedDict(
                {
                    "LoadBalancerArn": self.LoadBalancerArn,
                }
            )
        except AttributeError:
            return None
        return LoadBalancer.objects.get(**pk)

    @cached_property
    def rules(self) -> Optional[List["Rule"]]:
        """
        Return the ARNs of :py:class:`Rule` objects that belong to this
        listener, if any.
        """

        try:
            pk = OrderedDict(
                {
                    "ListenerArn": self.ListenerArn,
                }
            )
        except AttributeError:
            return []
        return Rule.objects.list(**pk)


class HostHeaderConditionConfig(Boto3Model):
    """
    Information for a host header condition.

    Specify only when ``Field`` is ``host-
    header``.
    """

    #: The host names. The maximum size of each name is 128 characters. The comparison
    #: is case insensitive. The following wildcard characters are supported: \*
    #: (matches 0 or more characters) and ? (matches exactly 1 character).
    Values: Optional[List[str]] = None


class PathPatternConditionConfig(Boto3Model):
    """
    Information for a path pattern condition.

    Specify only when ``Field`` is
    ``path-pattern``.
    """

    #: The path patterns to compare against the request URL. The maximum size of each
    #: string is 128 characters. The comparison is case sensitive. The following
    #: wildcard characters are supported: \* (matches 0 or more characters) and ?
    #: (matches exactly 1 character).
    Values: Optional[List[str]] = None


class HttpHeaderConditionConfig(Boto3Model):
    """
    Information for an HTTP header condition.

    Specify only when ``Field`` is
    ``http-header``.
    """

    #: The name of the HTTP header field. The maximum size is 40 characters. The
    #: header name is case insensitive. The allowed characters are specified by RFC
    #: 7230. Wildcards are not supported.
    HttpHeaderName: Optional[str] = None
    #: The strings to compare against the value of the HTTP header. The maximum size
    #: of each string is 128 characters. The comparison strings are case insensitive.
    #: The following wildcard characters are supported: \* (matches 0 or more
    #: characters) and ? (matches exactly 1 character).
    Values: Optional[List[str]] = None


class QueryStringKeyValuePair(Boto3Model):
    """
    Information about a key/value pair.
    """

    #: The key. You can omit the key.
    Key: Optional[str] = None
    #: The value.
    Value: Optional[str] = None


class QueryStringConditionConfig(Boto3Model):
    """
    Information for a query string condition.

    Specify only when ``Field`` is
    ``query-string``.
    """

    #: The key/value pairs or values to find in the query string. The maximum size of
    #: each string is 128 characters. The comparison is case insensitive. The
    #: following wildcard characters are supported: \* (matches 0 or more characters)
    #: and ? (matches exactly 1 character). To search for a literal '\*' or '?'
    #: character in a query string, you must escape these characters in ``Values``
    #: using a '\' character.
    Values: Optional[List["QueryStringKeyValuePair"]] = None


class HttpRequestMethodConditionConfig(Boto3Model):
    """
    Information for an HTTP method condition.

    Specify only when ``Field`` is
    ``http-request-method``.
    """

    #: The name of the request method. The maximum size is 40 characters. The allowed
    #: characters are A-Z, hyphen (-), and underscore (\_). The comparison is case
    #: sensitive. Wildcards are not supported; therefore, the method name must be an
    #: exact match.
    Values: Optional[List[str]] = None


class SourceIpConditionConfig(Boto3Model):
    """
    Information for a source IP condition.

    Specify only when ``Field`` is ``source-
    ip``.
    """

    #: The source IP addresses, in CIDR format. You can use both IPv4 and IPv6
    #: addresses. Wildcards are not supported.
    Values: Optional[List[str]] = None


class RuleCondition(Boto3Model):
    """
    Information about a condition for a rule.

    Each rule can optionally include up to one of each of the following conditions:
    ``http-request-method``, ``host-header``, ``path-pattern``, and ``source-ip``.
    Each rule can also optionally include one or more of each of the following
    conditions: ``http-header`` and ``query-string``. Note that the value for a
    condition cannot be empty.

    For more information, see `Quotas for your Application Load Balancers <https://
    docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-
    limits.html>`_.
    """

    #: The field in the HTTP request. The following are the possible values:
    Field: Optional[str] = None
    #: The condition value. Specify only when ``Field`` is ``host-header`` or ``path-
    #: pattern``. Alternatively, to specify multiple host names or multiple path
    #: patterns, use ``HostHeaderConfig`` or ``PathPatternConfig``.
    Values: Optional[List[str]] = None
    #: Information for a host header condition. Specify only when ``Field`` is ``host-
    #: header``.
    HostHeaderConfig: Optional[HostHeaderConditionConfig] = None
    #: Information for a path pattern condition. Specify only when ``Field`` is
    #: ``path-pattern``.
    PathPatternConfig: Optional[PathPatternConditionConfig] = None
    #: Information for an HTTP header condition. Specify only when ``Field`` is
    #: ``http-header``.
    HttpHeaderConfig: Optional[HttpHeaderConditionConfig] = None
    #: Information for a query string condition. Specify only when ``Field`` is
    #: ``query-string``.
    QueryStringConfig: Optional[QueryStringConditionConfig] = None
    #: Information for an HTTP method condition. Specify only when ``Field`` is
    #: ``http-request-method``.
    HttpRequestMethodConfig: Optional[HttpRequestMethodConditionConfig] = None
    #: Information for a source IP condition. Specify only when ``Field`` is ``source-
    #: ip``.
    SourceIpConfig: Optional[SourceIpConditionConfig] = None


class Rule(PrimaryBoto3Model):
    """
    Information about a rule.
    """

    objects: ClassVar[Boto3ModelManager] = RuleManager()

    #: The priority.
    Priority: Optional[str] = "1"
    #: The conditions. Each rule can include zero or one of the following conditions:
    #: ``http-request-method``, ``host-header``, ``path-pattern``, and ``source-ip``,
    #: and zero or more of the following conditions: ``http-header`` and ``query-
    #: string``.
    Conditions: List["RuleCondition"]
    #: The actions. Each rule must include exactly one of the following types of
    #: actions: ``forward``, ``redirect``, or ``fixed-response``, and it must be the
    #: last action to be performed.
    Actions: List["Action"]
    #: Indicates whether this is the default rule.
    IsDefault: Optional[bool] = False
    #: The Amazon Resource Name (ARN) of the rule.
    RuleArn: Optional[str] = None

    @property
    def pk(self) -> Optional[str]:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`RuleArn` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.RuleArn

    @property
    def arn(self) -> Optional[str]:
        """
        Return the ARN of the model.   This is the value of the
        :py:attr:`RuleArn` attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.RuleArn


class ResponseCodeMatcher(Boto3Model):
    """
    The HTTP or gRPC codes to use when checking for a successful response from
    a target.
    """

    #: For Application Load Balancers, you can specify values between 200 and 499,
    #: with the default value being 200. You can specify multiple values (for example,
    #: "200,202") or a range of values (for example, "200-299").
    HttpCode: Optional[str] = None
    #: You can specify values between 0 and 99. You can specify multiple values (for
    #: example, "0,1") or a range of values (for example, "0-5"). The default value is
    #: 12.
    GrpcCode: Optional[str] = None


class TargetGroup(PrimaryBoto3Model):
    """
    Information about a target group.
    """

    objects: ClassVar[Boto3ModelManager] = TargetGroupManager()

    #: The name of the target group.
    TargetGroupName: str
    #: The protocol to use for routing traffic to the targets.
    Protocol: Optional[
        Literal["HTTP", "HTTPS", "TCP", "TLS", "UDP", "TCP_UDP", "GENEVE"]
    ] = "HTTPS"
    #: The port on which the targets are listening. This parameter is not used if the
    #: target is a Lambda function.
    Port: int
    #: The ID of the VPC for the targets.
    VpcId: str
    #: The type of target that you must specify when registering targets with this
    #: target group. The possible values are ``instance`` (register targets by
    #: instance ID), ``ip`` (register targets by IP address), ``lambda`` (register a
    #: single Lambda function as a target), or ``alb`` (register a single Application
    #: Load Balancer as a target).
    TargetType: Optional[Literal["instance", "ip", "lambda", "alb"]] = "ip"
    #: The type of IP address used for this target group. The possible values are
    #: ``ipv4`` and ``ipv6``. This is an optional parameter. If not specified, the IP
    #: address type defaults to ``ipv4``.
    IpAddressType: Optional[Literal["ipv4", "ipv6"]] = "ipv4"
    #: The Amazon Resource Name (ARN) of the target group.
    TargetGroupArn: Optional[str] = None
    #: The protocol to use to connect with the target. The GENEVE, TLS, UDP, and
    #: TCP\_UDP protocols are not supported for health checks.
    HealthCheckProtocol: Optional[
        Literal["HTTP", "HTTPS", "TCP", "TLS", "UDP", "TCP_UDP", "GENEVE"]
    ] = None
    #: The port to use to connect with the target.
    HealthCheckPort: Optional[str] = None
    #: Indicates whether health checks are enabled.
    HealthCheckEnabled: Optional[bool] = None
    #: The approximate amount of time, in seconds, between health checks of an
    #: individual target.
    HealthCheckIntervalSeconds: Optional[int] = None
    #: The amount of time, in seconds, during which no response means a failed health
    #: check.
    HealthCheckTimeoutSeconds: Optional[int] = None
    #: The number of consecutive health checks successes required before considering
    #: an unhealthy target healthy.
    HealthyThresholdCount: Optional[int] = None
    #: The number of consecutive health check failures required before considering the
    #: target unhealthy.
    UnhealthyThresholdCount: Optional[int] = None
    #: The destination for health checks on the targets.
    HealthCheckPath: Optional[str] = None
    #: The HTTP or gRPC codes to use when checking for a successful response from a
    #: target.
    Matcher: Optional[ResponseCodeMatcher] = None
    #: The Amazon Resource Name (ARN) of the load balancer that routes traffic to this
    #: target group. You can use each target group with only one load balancer.
    LoadBalancerArns: Optional[List[str]] = None
    #: [HTTP/HTTPS protocol] The protocol version. The possible values are ``GRPC``,
    #: ``HTTP1``, and ``HTTP2``.
    ProtocolVersion: Optional[str] = None

    @property
    def pk(self) -> Optional[str]:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`TargetGroupArn` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.TargetGroupArn

    @property
    def arn(self) -> Optional[str]:
        """
        Return the ARN of the model.   This is the value of the
        :py:attr:`TargetGroupArn` attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.TargetGroupArn

    @property
    def name(self) -> Optional[str]:
        """
        Return the name of the model.   This is the value of the
        :py:attr:`TargetGroupName` attribute.

        Returns:
            The name of the model instance.
        """
        return self.TargetGroupName

    @cached_property
    def load_balancers(self) -> Optional[List["LoadBalancer"]]:
        """
        Return the ARNs of :py:class:`LoadBalancer` objects that this target
        group belongs to, if any.
        """

        try:
            pk = OrderedDict(
                {
                    "LoadBalancerArns": self.LoadBalancerArns,
                }
            )
        except AttributeError:
            return []
        return LoadBalancer.objects.list(**pk)

    @cached_property
    def vpc(self) -> Optional["Vpc"]:
        """
        Return the :py:class:`Vpc` object that this target group belongs to, if
        any.
        """

        try:
            pk = OrderedDict(
                {
                    "VpcId": self.VpcId,
                }
            )
        except AttributeError:
            return None
        return Vpc.objects.get(**pk)


# =======================
# Request/Response Models
# =======================


class SubnetMapping(Boto3Model):
    """
    Information about a subnet mapping.
    """

    #: The ID of the subnet.
    SubnetId: str
    #: [Network Load Balancers] The allocation ID of the Elastic IP address for an
    #: internet-facing load balancer.
    AllocationId: Optional[str] = None
    #: [Network Load Balancers] The private IPv4 address for an internal load
    #: balancer.
    PrivateIPv4Address: Optional[str] = None
    #: [Network Load Balancers] The IPv6 address.
    IPv6Address: Optional[str] = None


class CreateLoadBalancerOutput(Boto3Model):
    #: Information about the load balancer.
    LoadBalancers: Optional[List["LoadBalancer"]] = None


class DeleteLoadBalancerOutput(Boto3Model):
    pass


class DescribeLoadBalancersOutput(Boto3Model):
    #: Information about the load balancers.
    LoadBalancers: Optional[List["LoadBalancer"]] = None
    #: If there are additional results, this is the marker for the next set of
    #: results. Otherwise, this is null.
    NextMarker: Optional[str] = None


class CreateListenerOutput(Boto3Model):
    #: Information about the listener.
    Listeners: Optional[List["Listener"]] = None


class ModifyListenerOutput(Boto3Model):
    #: Information about the modified listener.
    Listeners: Optional[List["Listener"]] = None


class DeleteListenerOutput(Boto3Model):
    pass


class DescribeListenersOutput(Boto3Model):
    #: Information about the listeners.
    Listeners: Optional[List["Listener"]] = None
    #: If there are additional results, this is the marker for the next set of
    #: results. Otherwise, this is null.
    NextMarker: Optional[str] = None


class CreateRuleOutput(Boto3Model):
    #: Information about the rule.
    Rules: Optional[List["Rule"]] = None


class ModifyRuleOutput(Boto3Model):
    #: Information about the modified rule.
    Rules: Optional[List["Rule"]] = None


class DeleteRuleOutput(Boto3Model):
    pass


class DescribeRulesOutput(Boto3Model):
    #: Information about the rules.
    Rules: Optional[List["Rule"]] = None
    #: If there are additional results, this is the marker for the next set of
    #: results. Otherwise, this is null.
    NextMarker: Optional[str] = None


class CreateTargetGroupOutput(Boto3Model):
    #: Information about the target group.
    TargetGroups: Optional[List["TargetGroup"]] = None


class ModifyTargetGroupOutput(Boto3Model):
    #: Information about the modified target group.
    TargetGroups: Optional[List["TargetGroup"]] = None


class DeleteTargetGroupOutput(Boto3Model):
    pass


class DescribeTargetGroupsOutput(Boto3Model):
    #: Information about the target groups.
    TargetGroups: Optional[List["TargetGroup"]] = None
    #: If there are additional results, this is the marker for the next set of
    #: results. Otherwise, this is null.
    NextMarker: Optional[str] = None
