# This file is automatically generated by botocraft.  Do not edit directly.
# pylint: disable=anomalous-backslash-in-string,unsubscriptable-object,line-too-long,arguments-differ,arguments-renamed
# mypy: disable-error-code="index, override, assignment"
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Literal, Optional, Type, cast

from pydantic import Field

from botocraft.mixins.tags import TagsDictMixin
from botocraft.services.common import Filter, Tag

from .abstract import (Boto3Model, Boto3ModelManager, PrimaryBoto3Model,
                       ReadonlyBoto3Model, ReadonlyBoto3ModelManager,
                       ReadonlyPrimaryBoto3Model)

# ===============
# Managers
# ===============


class DBInstanceManager(Boto3ModelManager):
    service_name: str = "rds"

    def create(
        self,
        model: "DBInstance",
        MasterUserPassword: Optional[str] = None,
        VpcSecurityGroupIds: Optional[List[str]] = None,
        DBSubnetGroupName: Optional[str] = None,
        DBParameterGroupName: Optional[str] = None,
        OptionGroupName: Optional[str] = None,
        TdeCredentialPassword: Optional[str] = None,
        Domain: Optional[str] = None,
        DomainFqdn: Optional[str] = None,
        DomainOu: Optional[str] = None,
        DomainAuthSecretArn: Optional[str] = None,
        DomainDnsIps: Optional[List[str]] = None,
        DomainIAMRoleName: Optional[str] = None,
        ManageMasterUserPassword: Optional[bool] = None,
        MasterUserSecretKmsKeyId: Optional[str] = None,
    ) -> "DBInstance":
        """
        Creates a new DB instance.

        Args:
            model: The :py:class:`DBInstance` to create.

        Keyword Args:
            MasterUserPassword: The password for the master user.
            VpcSecurityGroupIds: A list of Amazon EC2 VPC security groups to associate
                with this DB instance.
            DBSubnetGroupName: A DB subnet group to associate with this DB instance.
            DBParameterGroupName: The name of the DB parameter group to associate with
                this DB instance. If you don't specify a value, then Amazon RDS uses the
                default DB parameter group for the specified DB engine and version.
            OptionGroupName: The option group to associate the DB instance with.
            TdeCredentialPassword: The password for the given ARN from the key store in
                order to access the device.
            Domain: The Active Directory directory ID to create the DB instance in.
                Currently, only Microsoft SQL Server, MySQL, Oracle, and PostgreSQL DB
                instances can be created in an Active Directory Domain.
            DomainFqdn: The fully qualified domain name (FQDN) of an Active Directory
                domain.
            DomainOu: The Active Directory organizational unit for your DB instance to
                join.
            DomainAuthSecretArn: The ARN for the Secrets Manager secret with the
                credentials for the user joining the domain.
            DomainDnsIps: The IPv4 DNS IP addresses of your primary and secondary
                Active Directory domain controllers.
            DomainIAMRoleName: The name of the IAM role to use when making API calls to
                the Directory Service.
            ManageMasterUserPassword: Specifies whether to manage the master user
                password with Amazon Web Services Secrets Manager.
            MasterUserSecretKmsKeyId: The Amazon Web Services KMS key identifier to
                encrypt a secret that is automatically generated and managed in Amazon Web
                Services Secrets Manager.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            DBInstanceIdentifier=data.get("DBInstanceIdentifier"),
            DBInstanceClass=data.get("DBInstanceClass"),
            Engine=data.get("Engine"),
            DBName=data.get("DBName"),
            AllocatedStorage=data.get("AllocatedStorage"),
            MasterUsername=data.get("MasterUsername"),
            MasterUserPassword=self.serialize(MasterUserPassword),
            VpcSecurityGroupIds=self.serialize(VpcSecurityGroupIds),
            AvailabilityZone=data.get("AvailabilityZone"),
            DBSubnetGroupName=self.serialize(DBSubnetGroupName),
            PreferredMaintenanceWindow=data.get("PreferredMaintenanceWindow"),
            DBParameterGroupName=self.serialize(DBParameterGroupName),
            BackupRetentionPeriod=data.get("BackupRetentionPeriod"),
            PreferredBackupWindow=data.get("PreferredBackupWindow"),
            Port=data["DbInstancePort"],
            MultiAZ=data.get("MultiAZ"),
            EngineVersion=data.get("EngineVersion"),
            AutoMinorVersionUpgrade=data.get("AutoMinorVersionUpgrade"),
            LicenseModel=data.get("LicenseModel"),
            Iops=data.get("Iops"),
            OptionGroupName=self.serialize(OptionGroupName),
            CharacterSetName=data.get("CharacterSetName"),
            NcharCharacterSetName=data.get("NcharCharacterSetName"),
            PubliclyAccessible=data.get("PubliclyAccessible"),
            Tags=data["TagList"],
            DBClusterIdentifier=data.get("DBClusterIdentifier"),
            StorageType=data.get("StorageType"),
            TdeCredentialArn=data.get("TdeCredentialArn"),
            TdeCredentialPassword=self.serialize(TdeCredentialPassword),
            StorageEncrypted=data.get("StorageEncrypted"),
            KmsKeyId=data.get("KmsKeyId"),
            Domain=self.serialize(Domain),
            DomainFqdn=self.serialize(DomainFqdn),
            DomainOu=self.serialize(DomainOu),
            DomainAuthSecretArn=self.serialize(DomainAuthSecretArn),
            DomainDnsIps=self.serialize(DomainDnsIps),
            CopyTagsToSnapshot=data.get("CopyTagsToSnapshot"),
            MonitoringInterval=data.get("MonitoringInterval"),
            MonitoringRoleArn=data.get("MonitoringRoleArn"),
            DomainIAMRoleName=self.serialize(DomainIAMRoleName),
            PromotionTier=data.get("PromotionTier"),
            Timezone=data.get("Timezone"),
            EnableIAMDatabaseAuthentication=data["IAMDatabaseAuthenticationEnabled"],
            EnablePerformanceInsights=data["PerformanceInsightsEnabled"],
            PerformanceInsightsKMSKeyId=data.get("PerformanceInsightsKMSKeyId"),
            PerformanceInsightsRetentionPeriod=data.get(
                "PerformanceInsightsRetentionPeriod"
            ),
            EnableCloudwatchLogsExports=data["EnabledCloudwatchLogsExports"],
            ProcessorFeatures=data.get("ProcessorFeatures"),
            DeletionProtection=data.get("DeletionProtection"),
            MaxAllocatedStorage=data.get("MaxAllocatedStorage"),
            EnableCustomerOwnedIp=data["CustomerOwnedIpEnabled"],
            CustomIamInstanceProfile=data.get("CustomIamInstanceProfile"),
            BackupTarget=data.get("BackupTarget"),
            NetworkType=data.get("NetworkType"),
            StorageThroughput=data.get("StorageThroughput"),
            ManageMasterUserPassword=self.serialize(ManageMasterUserPassword),
            MasterUserSecretKmsKeyId=self.serialize(MasterUserSecretKmsKeyId),
            CACertificateIdentifier=data.get("CACertificateIdentifier"),
            DBSystemId=data.get("DBSystemId"),
        )
        _response = self.client.create_db_instance(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateDBInstanceResult(**_response)

        return cast("DBInstance", response.DBInstance)

    def update(
        self,
        model: "DBInstance",
        DBSubnetGroupName: Optional[str] = None,
        VpcSecurityGroupIds: Optional[List[str]] = None,
        ApplyImmediately: Optional[bool] = None,
        MasterUserPassword: Optional[str] = None,
        DBParameterGroupName: Optional[str] = None,
        AllowMajorVersionUpgrade: Optional[bool] = None,
        OptionGroupName: Optional[str] = None,
        NewDBInstanceIdentifier: Optional[str] = None,
        TdeCredentialPassword: Optional[str] = None,
        Domain: Optional[str] = None,
        DomainFqdn: Optional[str] = None,
        DomainOu: Optional[str] = None,
        DomainAuthSecretArn: Optional[str] = None,
        DomainDnsIps: Optional[List[str]] = None,
        DBPortNumber: Optional[int] = None,
        DomainIAMRoleName: Optional[str] = None,
        DisableDomain: Optional[bool] = None,
        CloudwatchLogsExportConfiguration: Optional[
            "CloudwatchLogsExportConfiguration"
        ] = None,
        UseDefaultProcessorFeatures: Optional[bool] = None,
        CertificateRotationRestart: Optional[bool] = None,
        ResumeFullAutomationModeMinutes: Optional[int] = None,
        ManageMasterUserPassword: Optional[bool] = None,
        RotateMasterUserPassword: Optional[bool] = None,
        MasterUserSecretKmsKeyId: Optional[str] = None,
    ) -> "DBInstance":
        """
        Modifies settings for a DB instance. You can change one or more
        database configuration parameters by specifying these parameters and
        the new values in the request. To learn what modifications you can make
        to your DB instance, call ``DescribeValidDBInstanceModifications``
        before you call ``ModifyDBInstance``.

        Args:
            model: The :py:class:`DBInstance` to update.

        Keyword Args:
            DBSubnetGroupName: The new DB subnet group for the DB instance. You can use
                this parameter to move your DB instance to a different VPC. If your DB
                instance isn't in a VPC, you can also use this parameter to move your DB
                instance into a VPC. For more information, see `Working with a DB instance
                in a VPC <https://docs.aws.amazon. com/AmazonRDS/latest/UserGuide/USER_VPC.
                WorkingWithRDSInstanceinaVPC.html#USER_ VPC.Non-VPC2VPC>`_ in the *Amazon
                RDS User Guide*.
            VpcSecurityGroupIds: A list of Amazon EC2 VPC security groups to associate
                with this DB instance. This change is asynchronously applied as soon as
                possible.
            ApplyImmediately: Specifies whether the modifications in this request and
                any pending modifications are asynchronously applied as soon as possible,
                regardless of the ``PreferredMaintenanceWindow`` setting for the DB
                instance. By default, this parameter is disabled.
            MasterUserPassword: The new password for the master user.
            DBParameterGroupName: The name of the DB parameter group to apply to the DB
                instance.
            AllowMajorVersionUpgrade: Specifies whether major version upgrades are
                allowed. Changing this parameter doesn't result in an outage and the change
                is asynchronously applied as soon as possible.
            OptionGroupName: The option group to associate the DB instance with.
            NewDBInstanceIdentifier: The new identifier for the DB instance when
                renaming a DB instance. When you change the DB instance identifier, an
                instance reboot occurs immediately if you enable ``ApplyImmediately``, or
                will occur during the next maintenance window if you disable
                ``ApplyImmediately``. This value is stored as a lowercase string.
            TdeCredentialPassword: The password for the given ARN from the key store in
                order to access the device.
            Domain: The Active Directory directory ID to move the DB instance to.
                Specify ``none`` to remove the instance from its current domain. You must
                create the domain before this operation. Currently, you can create only
                MySQL, Microsoft SQL Server, Oracle, and PostgreSQL DB instances in an
                Active Directory Domain.
            DomainFqdn: The fully qualified domain name (FQDN) of an Active Directory
                domain.
            DomainOu: The Active Directory organizational unit for your DB instance to
                join.
            DomainAuthSecretArn: The ARN for the Secrets Manager secret with the
                credentials for the user joining the domain.
            DomainDnsIps: The IPv4 DNS IP addresses of your primary and secondary
                Active Directory domain controllers.
            DBPortNumber: The port number on which the database accepts connections.
            DomainIAMRoleName: The name of the IAM role to use when making API calls to
                the Directory Service.
            DisableDomain: Specifies whether to remove the DB instance from the Active
                Directory domain.
            CloudwatchLogsExportConfiguration: The log types to be enabled for export
                to CloudWatch Logs for a specific DB instance.
            UseDefaultProcessorFeatures: Specifies whether the DB instance class of the
                DB instance uses its default processor features.
            CertificateRotationRestart: Specifies whether the DB instance is restarted
                when you rotate your SSL/TLS certificate.
            ResumeFullAutomationModeMinutes: The number of minutes to pause the
                automation. When the time period ends, RDS Custom resumes full automation.
            ManageMasterUserPassword: Specifies whether to manage the master user
                password with Amazon Web Services Secrets Manager.
            RotateMasterUserPassword: Specifies whether to rotate the secret managed by
                Amazon Web Services Secrets Manager for the master user password.
            MasterUserSecretKmsKeyId: The Amazon Web Services KMS key identifier to
                encrypt a secret that is automatically generated and managed in Amazon Web
                Services Secrets Manager.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            DBInstanceIdentifier=data.get("DBInstanceIdentifier"),
            AllocatedStorage=data.get("AllocatedStorage"),
            DBInstanceClass=data.get("DBInstanceClass"),
            DBSubnetGroupName=self.serialize(DBSubnetGroupName),
            VpcSecurityGroupIds=self.serialize(VpcSecurityGroupIds),
            ApplyImmediately=self.serialize(ApplyImmediately),
            MasterUserPassword=self.serialize(MasterUserPassword),
            DBParameterGroupName=self.serialize(DBParameterGroupName),
            BackupRetentionPeriod=data.get("BackupRetentionPeriod"),
            PreferredBackupWindow=data.get("PreferredBackupWindow"),
            PreferredMaintenanceWindow=data.get("PreferredMaintenanceWindow"),
            MultiAZ=data.get("MultiAZ"),
            EngineVersion=data.get("EngineVersion"),
            AllowMajorVersionUpgrade=self.serialize(AllowMajorVersionUpgrade),
            AutoMinorVersionUpgrade=data.get("AutoMinorVersionUpgrade"),
            LicenseModel=data.get("LicenseModel"),
            Iops=data.get("Iops"),
            OptionGroupName=self.serialize(OptionGroupName),
            NewDBInstanceIdentifier=self.serialize(NewDBInstanceIdentifier),
            StorageType=data.get("StorageType"),
            TdeCredentialArn=data.get("TdeCredentialArn"),
            TdeCredentialPassword=self.serialize(TdeCredentialPassword),
            CACertificateIdentifier=data.get("CACertificateIdentifier"),
            Domain=self.serialize(Domain),
            DomainFqdn=self.serialize(DomainFqdn),
            DomainOu=self.serialize(DomainOu),
            DomainAuthSecretArn=self.serialize(DomainAuthSecretArn),
            DomainDnsIps=self.serialize(DomainDnsIps),
            CopyTagsToSnapshot=data.get("CopyTagsToSnapshot"),
            MonitoringInterval=data.get("MonitoringInterval"),
            DBPortNumber=self.serialize(DBPortNumber),
            PubliclyAccessible=data.get("PubliclyAccessible"),
            MonitoringRoleArn=data.get("MonitoringRoleArn"),
            DomainIAMRoleName=self.serialize(DomainIAMRoleName),
            DisableDomain=self.serialize(DisableDomain),
            PromotionTier=data.get("PromotionTier"),
            EnableIAMDatabaseAuthentication=data["IAMDatabaseAuthenticationEnabled"],
            EnablePerformanceInsights=data["PerformanceInsightsEnabled"],
            PerformanceInsightsKMSKeyId=data.get("PerformanceInsightsKMSKeyId"),
            PerformanceInsightsRetentionPeriod=data.get(
                "PerformanceInsightsRetentionPeriod"
            ),
            CloudwatchLogsExportConfiguration=self.serialize(
                CloudwatchLogsExportConfiguration
            ),
            ProcessorFeatures=data.get("ProcessorFeatures"),
            UseDefaultProcessorFeatures=self.serialize(UseDefaultProcessorFeatures),
            DeletionProtection=data.get("DeletionProtection"),
            MaxAllocatedStorage=data.get("MaxAllocatedStorage"),
            CertificateRotationRestart=self.serialize(CertificateRotationRestart),
            ReplicaMode=data.get("ReplicaMode"),
            EnableCustomerOwnedIp=data["CustomerOwnedIpEnabled"],
            AwsBackupRecoveryPointArn=data.get("AwsBackupRecoveryPointArn"),
            AutomationMode=data.get("AutomationMode"),
            ResumeFullAutomationModeMinutes=self.serialize(
                ResumeFullAutomationModeMinutes
            ),
            NetworkType=data.get("NetworkType"),
            StorageThroughput=data.get("StorageThroughput"),
            ManageMasterUserPassword=self.serialize(ManageMasterUserPassword),
            RotateMasterUserPassword=self.serialize(RotateMasterUserPassword),
            MasterUserSecretKmsKeyId=self.serialize(MasterUserSecretKmsKeyId),
            Engine=data.get("Engine"),
        )
        _response = self.client.modify_db_instance(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ModifyDBInstanceResult(**_response)

        return cast("DBInstance", response.DBInstance)

    def delete(
        self,
        DBInstanceIdentifier: str,
        *,
        SkipFinalSnapshot: Optional[bool] = None,
        FinalDBSnapshotIdentifier: Optional[str] = None,
        DeleteAutomatedBackups: Optional[bool] = None
    ) -> "DBInstance":
        """
        The DeleteDBInstance action deletes a previously provisioned DB
        instance. When you delete a DB instance, all automated backups for that
        instance are deleted and can't be recovered. Manual DB snapshots of the
        DB instance to be deleted by ``DeleteDBInstance`` are not deleted.

        Args:
            DBInstanceIdentifier: The DB instance identifier for the DB instance to be
                deleted. This parameter isn't case-sensitive.

        Keyword Args:
            SkipFinalSnapshot: A value that indicates whether to skip the creation of a
                final DB snapshot before deleting the instance. If you enable this
                parameter, RDS doesn't create a DB snapshot. If you don't enable this
                parameter, RDS creates a DB snapshot before the DB instance is deleted. By
                default, skip isn't enabled, and the DB snapshot is created.
            FinalDBSnapshotIdentifier: The ``DBSnapshotIdentifier`` of the new
                ``DBSnapshot`` created when the ``SkipFinalSnapshot`` parameter is
                disabled.
            DeleteAutomatedBackups: A value that indicates whether to remove automated
                backups immediately after the DB instance is deleted. This parameter isn't
                case-sensitive. The default is to remove automated backups immediately
                after the DB instance is deleted.
        """
        args: Dict[str, Any] = dict(
            DBInstanceIdentifier=self.serialize(DBInstanceIdentifier),
            SkipFinalSnapshot=self.serialize(SkipFinalSnapshot),
            FinalDBSnapshotIdentifier=self.serialize(FinalDBSnapshotIdentifier),
            DeleteAutomatedBackups=self.serialize(DeleteAutomatedBackups),
        )
        _response = self.client.delete_db_instance(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DeleteDBInstanceResult(**_response)
        return cast(DBInstance, response.DBInstance)

    def get(
        self, DBInstanceIdentifier: str, *, Filters: Optional[List[Filter]] = None
    ) -> Optional["DBInstance"]:
        """
        Describes provisioned RDS instances. This API supports pagination.

        Args:
            DBInstanceIdentifier: The user-supplied instance identifier or the Amazon
                Resource Name (ARN) of the DB instance. If this parameter is specified,
                information from only the specific DB instance is returned. This parameter
                isn't case-sensitive.

        Keyword Args:
            Filters: A filter that specifies one or more DB instances to describe.
        """
        args: Dict[str, Any] = dict(
            DBInstanceIdentifier=self.serialize(DBInstanceIdentifier),
            Filters=self.serialize(Filters),
        )
        _response = self.client.describe_db_instances(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DBInstanceMessage(**_response)

        if response.DBInstances:
            return response.DBInstances[0]
        return None

    def list(
        self,
        *,
        DBInstanceIdentifier: Optional[str] = None,
        Filters: Optional[List[Filter]] = None
    ) -> List["DBInstance"]:
        """
        Describes provisioned RDS instances. This API supports pagination.

        Keyword Args:
            DBInstanceIdentifier: The user-supplied instance identifier or the Amazon
                Resource Name (ARN) of the DB instance. If this parameter is specified,
                information from only the specific DB instance is returned. This parameter
                isn't case-sensitive.
            Filters: A filter that specifies one or more DB instances to describe.
        """
        paginator = self.client.get_paginator("describe_db_instances")
        args: Dict[str, Any] = dict(
            DBInstanceIdentifier=self.serialize(DBInstanceIdentifier),
            Filters=self.serialize(Filters),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results: List["DBInstance"] = []
        for _response in response_iterator:
            response = DBInstanceMessage(**_response)
            if response.DBInstances:
                results.extend(response.DBInstances)
            else:
                break
        return results


# ==============
# Service Models
# ==============


class RDSEndpoint(Boto3Model):
    """
    The connection endpoint for the DB instance.

    The endpoint might not be shown for instances with the status of ``creating``.
    """

    #: Specifies the DNS address of the DB instance.
    Address: Optional[str] = None
    #: Specifies the port that the database engine is listening on.
    Port: Optional[int] = None
    #: Specifies the ID that Amazon Route 53 assigns when you create a hosted zone.
    HostedZoneId: Optional[str] = None


class DBSecurityGroupMembership(Boto3Model):
    """This data type is used as a response element in the following actions:

    * ``ModifyDBInstance``
    * ``RebootDBInstance``
    * ``RestoreDBInstanceFromDBSnapshot``
    * ``RestoreDBInstanceToPointInTime``"""

    #: The name of the DB security group.
    DBSecurityGroupName: Optional[str] = None
    #: The status of the DB security group.
    Status: Optional[str] = None


class VpcSecurityGroupMembership(Boto3Model):
    """
    This data type is used as a response element for queries on VPC security
    group membership.
    """

    #: The name of the VPC security group.
    VpcSecurityGroupId: Optional[str] = None
    #: The membership status of the VPC security group.
    Status: Optional[str] = None


class DBParameterGroupStatus(Boto3Model):
    """
    The status of the DB parameter group.

    This data type is used as a response element in the following actions:

    * ``CreateDBInstance``
    * ``CreateDBInstanceReadReplica``
    * ``DeleteDBInstance``
    * ``ModifyDBInstance``
    * ``RebootDBInstance``
    * ``RestoreDBInstanceFromDBSnapshot``
    """

    #: The name of the DB parameter group.
    DBParameterGroupName: Optional[str] = None
    #: The status of parameter updates.
    ParameterApplyStatus: Optional[str] = None


class RDSAvailabilityZone(Boto3Model):
    """
    Contains Availability Zone information.

    This data type is used as an element in the ``OrderableDBInstanceOption`` data
    type.
    """

    #: The name of the Availability Zone.
    Name: Optional[str] = None


class Outpost(Boto3Model):
    """
    If the subnet is associated with an Outpost, this value specifies the
    Outpost.

    For more information about RDS on Outposts, see
    `Amazon RDS on Amazon Web Services Outposts <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/rds-
    on-outposts.html>`_ in the *Amazon RDS User Guide.*
    """

    #: The Amazon Resource Name (ARN) of the Outpost.
    Arn: Optional[str] = None


class RDSSubnet(Boto3Model):
    """
    This data type is used as a response element for the
    ``DescribeDBSubnetGroups`` operation.
    """

    #: The identifier of the subnet.
    SubnetIdentifier: Optional[str] = None
    #: Contains Availability Zone information.
    SubnetAvailabilityZone: Optional[RDSAvailabilityZone] = None
    #: If the subnet is associated with an Outpost, this value specifies the Outpost.
    SubnetOutpost: Optional[Outpost] = None
    #: The status of the subnet.
    SubnetStatus: Optional[str] = None


class RDSDBSubnetGroup(Boto3Model):
    """
    Information about the subnet group associated with the DB instance,
    including the name, description, and subnets in the subnet group.
    """

    #: The name of the DB subnet group.
    DBSubnetGroupName: Optional[str] = None
    #: Provides the description of the DB subnet group.
    DBSubnetGroupDescription: Optional[str] = None
    #: Provides the VpcId of the DB subnet group.
    VpcId: Optional[str] = None
    #: Provides the status of the DB subnet group.
    SubnetGroupStatus: Optional[str] = None
    #: Contains a list of ``Subnet`` elements.
    Subnets: Optional[List["RDSSubnet"]] = None
    #: The Amazon Resource Name (ARN) for the DB subnet group.
    DBSubnetGroupArn: Optional[str] = None
    #: The network type of the DB subnet group.
    SupportedNetworkTypes: Optional[List[str]] = None


class RDSPendingCloudwatchLogsExports(Boto3Model):
    """
    A list of the log types whose configuration is still pending.

    In other words, these log types are in the process of being activated or
    deactivated.
    """

    #: Log types that are in the process of being deactivated. After they are
    #: deactivated, these log types aren't exported to CloudWatch Logs.
    LogTypesToEnable: Optional[List[str]] = None
    #: Log types that are in the process of being enabled. After they are enabled,
    #: these log types are exported to CloudWatch Logs.
    LogTypesToDisable: Optional[List[str]] = None


class ProcessorFeature(Boto3Model):
    """
    Contains the processor features of a DB instance class.

    To specify the number of CPU cores, use the ``coreCount`` feature name for the
    ``Name`` parameter. To specify the number of threads per core, use the
    ``threadsPerCore`` feature name for the ``Name`` parameter.

    You can set the processor features of the DB instance class for a DB instance
    when you call one of the following actions:

    * ``CreateDBInstance``
    * ``ModifyDBInstance``
    * ``RestoreDBInstanceFromDBSnapshot``
    * ``RestoreDBInstanceFromS3``
    * ``RestoreDBInstanceToPointInTime``

    You can view the valid processor values for a particular instance class by
    calling the ``DescribeOrderableDBInstanceOptions`` action and specifying the
    instance class for the ``DBInstanceClass`` parameter.

    In addition, you can use the following actions for DB instance class processor
    information:

    * ``DescribeDBInstances``
    * ``DescribeDBSnapshots``
    * ``DescribeValidDBInstanceModifications``

    If you call ``DescribeDBInstances``, ``ProcessorFeature`` returns non-null
    values only if the following conditions are met:

    * You are accessing an Oracle DB instance.
    * Your Oracle DB instance class supports configuring the number of CPU cores
      and threads per core.
    * The current number CPU cores and threads is set to a non-default value.

    For more information, see `Configuring the Processor of the DB Instance Class <
    https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.DBInstanceClass
    .html#USER_ConfigureProcessor>`_ in the *Amazon RDS User Guide.*
    """

    #: The name of the processor feature. Valid names are ``coreCount`` and
    #: ``threadsPerCore``.
    Name: Optional[str] = None
    #: The value of a processor feature name.
    Value: Optional[str] = None


class RDSPendingModifiedValues(Boto3Model):
    """
    Information about pending changes to the DB instance.

    This information is returned only when there are pending changes. Specific
    changes are identified by subelements.
    """

    #: The name of the compute and memory capacity class for the DB instance.
    DBInstanceClass: Optional[str] = None
    #: The allocated storage size for the DB instance specified in gibibytes (GiB).
    AllocatedStorage: Optional[int] = None
    #: The master credentials for the DB instance.
    MasterUserPassword: Optional[str] = None
    #: The port for the DB instance.
    Port: Optional[int] = None
    #: The number of days for which automated backups are retained.
    BackupRetentionPeriod: Optional[int] = None
    #: A value that indicates that the Single-AZ DB instance will change to a Multi-AZ
    #: deployment.
    MultiAZ: Optional[bool] = None
    #: The database engine version.
    EngineVersion: Optional[str] = None
    #: The license model for the DB instance.
    LicenseModel: Optional[str] = None
    #: The Provisioned IOPS value for the DB instance.
    Iops: Optional[int] = None
    #: The database identifier for the DB instance.
    DBInstanceIdentifier: Optional[str] = None
    #: The storage type of the DB instance.
    StorageType: Optional[str] = None
    #: The identifier of the CA certificate for the DB instance.
    CACertificateIdentifier: Optional[str] = None
    #: The DB subnet group for the DB instance.
    DBSubnetGroupName: Optional[str] = None
    #: A list of the log types whose configuration is still pending. In other words,
    #: these log types are in the process of being activated or deactivated.
    PendingCloudwatchLogsExports: Optional[RDSPendingCloudwatchLogsExports] = None
    #: The number of CPU cores and the number of threads per core for the DB instance
    #: class of the DB instance.
    ProcessorFeatures: Optional[List["ProcessorFeature"]] = None
    #: Whether mapping of Amazon Web Services Identity and Access Management (IAM)
    #: accounts to database accounts is enabled.
    IAMDatabaseAuthenticationEnabled: Optional[bool] = None
    #: The automation mode of the RDS Custom DB instance: ``full`` or ``all-paused``.
    #: If ``full``, the DB instance automates monitoring and instance recovery. If
    #: ``all-paused``, the instance pauses automation for the duration set by
    #: ``--resume-full-automation-mode-minutes``.
    AutomationMode: Optional[Literal["full", "all-paused"]] = None
    #: The number of minutes to pause the automation. When the time period ends, RDS
    #: Custom resumes full automation. The minimum value is 60 (default). The maximum
    #: value is 1,440.
    ResumeFullAutomationModeTime: Optional[datetime] = None
    #: The storage throughput of the DB instance.
    StorageThroughput: Optional[int] = None
    #: The database engine of the DB instance.
    Engine: Optional[str] = None


class OptionGroupMembership(Boto3Model):
    """
    Provides information on the option groups the DB instance is a member of.
    """

    #: The name of the option group that the instance belongs to.
    OptionGroupName: Optional[str] = None
    #: The status of the DB instance's option group membership. Valid values are:
    #: ``in-sync``, ``pending-apply``, ``pending-removal``, ``pending-maintenance-
    #: apply``, ``pending-maintenance-removal``, ``applying``, ``removing``, and
    #: ``failed``.
    Status: Optional[str] = None


class DBInstanceStatusInfo(Boto3Model):
    """
    Provides a list of status information for a DB instance.
    """

    #: This value is currently "read replication."
    StatusType: Optional[str] = None
    #: Boolean value that is true if the instance is operating normally, or false if
    #: the instance is in an error state.
    Normal: Optional[bool] = None
    #: Status of the DB instance. For a StatusType of read replica, the values can be
    #: replicating, replication stop point set, replication stop point reached, error,
    #: stopped, or terminated.
    Status: Optional[str] = None
    #: Details of the error if there is an error for the instance. If the instance
    #: isn't in an error state, this value is blank.
    Message: Optional[str] = None


class DomainMembership(Boto3Model):
    """
    An Active Directory Domain membership record associated with the DB
    instance or cluster.
    """

    #: The identifier of the Active Directory Domain.
    Domain: Optional[str] = None
    #: The status of the Active Directory Domain membership for the DB instance or
    #: cluster. Values include ``joined``, ``pending-join``, ``failed``, and so on.
    Status: Optional[str] = None
    #: The fully qualified domain name (FQDN) of the Active Directory Domain.
    FQDN: Optional[str] = None
    #: The name of the IAM role used when making API calls to the Directory Service.
    IAMRoleName: Optional[str] = None
    #: The Active Directory organizational unit for the DB instance or cluster.
    OU: Optional[str] = None
    #: The ARN for the Secrets Manager secret with the credentials for the user that's
    #: a member of the domain.
    AuthSecretArn: Optional[str] = None
    #: The IPv4 DNS IP addresses of the primary and secondary Active Directory domain
    #: controllers.
    DnsIps: Optional[List[str]] = None


class DBInstanceRole(Boto3Model):
    """
    Describes an Amazon Web Services Identity and Access Management (IAM) role
    that is associated with a DB instance.
    """

    #: The Amazon Resource Name (ARN) of the IAM role that is associated with the DB
    #: instance.
    RoleArn: Optional[str] = None
    #: The name of the feature associated with the Amazon Web Services Identity and
    #: Access Management (IAM) role. For information about supported feature names,
    #: see ``DBEngineVersion``.
    FeatureName: Optional[str] = None
    #: Describes the state of association between the IAM role and the DB instance.
    #: The Status property returns one of the following values:
    Status: Optional[str] = None


class DBInstanceAutomatedBackupsReplication(Boto3Model):
    """
    Automated backups of a DB instance replicated to another Amazon Web
    Services Region.

    They consist of system backups, transaction logs, and database instance
    properties.
    """

    #: The Amazon Resource Name (ARN) of the replicated automated backups.
    DBInstanceAutomatedBackupsArn: Optional[str] = None


class RDSMasterUserSecret(Boto3Model):
    """
    The secret managed by RDS in Amazon Web Services Secrets Manager for the
    master user password.

    For more information, see
    `Password management with Amazon Web Services Secrets Manager <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/rds-secrets-
    manager.html>`_ in the *Amazon RDS User Guide.*
    """

    #: The Amazon Resource Name (ARN) of the secret.
    SecretArn: Optional[str] = None
    #: The status of the secret.
    SecretStatus: Optional[str] = None
    #: The Amazon Web Services KMS key identifier that is used to encrypt the secret.
    KmsKeyId: Optional[str] = None


class RDSCertificateDetails(Boto3Model):
    """
    The details of the DB instance's server certificate.
    """

    #: The CA identifier of the CA certificate used for the DB instance's server
    #: certificate.
    CAIdentifier: Optional[str] = None
    #: The expiration date of the DB instance’s server certificate.
    ValidTill: Optional[datetime] = None


class DBInstance(PrimaryBoto3Model):
    """
    Contains the details of an Amazon RDS DB instance.

    This data type is used as a response element in the operations
    ``CreateDBInstance``, ``CreateDBInstanceReadReplica``, ``DeleteDBInstance``,
    ``DescribeDBInstances``, ``ModifyDBInstance``, ``PromoteReadReplica``,
    ``RebootDBInstance``, ``RestoreDBInstanceFromDBSnapshot``,
    ``RestoreDBInstanceFromS3``, ``RestoreDBInstanceToPointInTime``,
    ``StartDBInstance``, and ``StopDBInstance``.
    """

    objects: ClassVar[Boto3ModelManager] = DBInstanceManager()

    #: The user-supplied database identifier. This identifier is the unique key that
    #: identifies a DB instance.
    DBInstanceIdentifier: str
    #: The name of the compute and memory capacity class of the DB instance.
    DBInstanceClass: str
    #: Indicates whether the DB instance is a Multi-AZ deployment. This setting
    #: doesn't apply to RDS Custom DB instances.
    MultiAZ: Optional[bool]
    #: The database engine used for this DB instance.
    Engine: str
    #: The version of the database engine.
    EngineVersion: str
    #: Indicates whether minor version patches are applied automatically.
    AutoMinorVersionUpgrade: Optional[bool] = True
    #: Indicates whether the DB instance is publicly accessible.
    PubliclyAccessible: Optional[bool]
    #: Indicates whether the DB instance is encrypted.
    StorageEncrypted: Optional[bool] = True
    #: The number of days for which automatic DB snapshots are retained.
    BackupRetentionPeriod: Optional[int] = 1
    #: The port that the DB instance listens on. If the DB instance is part of a DB
    #: cluster, this can be a different port than the DB cluster port.
    DbInstancePort: Optional[int] = None
    #: Indicates whether mapping of Amazon Web Services Identity and Access Management
    #: (IAM) accounts to database accounts is enabled for the DB instance.
    IAMDatabaseAuthenticationEnabled: Optional[bool] = None
    #: A list of log types that this DB instance is configured to export to CloudWatch
    #: Logs.
    EnabledCloudwatchLogsExports: Optional[List[str]] = None
    #: A list of tags. For more information, see `Tagging Amazon RDS Resources <https:
    #: //docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html>`_ in the
    #: *Amazon RDS User Guide.*
    TagList: Optional[List[Tag]] = None
    #: Indicates whether a customer-owned IP address (CoIP) is enabled for an RDS on
    #: Outposts DB instance.
    CustomerOwnedIpEnabled: Optional[bool] = None
    #: Indicates whether Performance Insights is enabled for the DB instance.
    PerformanceInsightsEnabled: Optional[bool] = None
    #: The current state of this database.
    DBInstanceStatus: str = Field(default=None, frozen=True)
    #: The time when a stopped DB instance is restarted automatically.
    AutomaticRestartTime: datetime = Field(default=None, frozen=True)
    #: The master username for the DB instance.
    MasterUsername: Optional[str] = None
    #: Contains the initial database name that you provided (if required) when you
    #: created the DB instance. This name is returned for the life of your DB
    #: instance. For an RDS for Oracle CDB instance, the name identifies the PDB
    #: rather than the CDB.
    DBName: Optional[str] = None
    #: The connection endpoint for the DB instance.
    Endpoint: RDSEndpoint = Field(default=None, frozen=True)
    #: The amount of storage in gibibytes (GiB) allocated for the DB instance.
    AllocatedStorage: Optional[int] = None
    #: The date and time when the DB instance was created.
    InstanceCreateTime: datetime = Field(default=None, frozen=True)
    #: The daily time range during which automated backups are created if automated
    #: backups are enabled, as determined by the ``BackupRetentionPeriod``.
    PreferredBackupWindow: Optional[str] = None
    #: A list of DB security group elements containing ``DBSecurityGroup.Name`` and
    #: ``DBSecurityGroup.Status`` subelements.
    DBSecurityGroups: Optional[List["DBSecurityGroupMembership"]] = None
    #: The list of Amazon EC2 VPC security groups that the DB instance belongs to.
    VpcSecurityGroups: List["VpcSecurityGroupMembership"] = Field(
        default=None, frozen=True
    )
    #: The list of DB parameter groups applied to this DB instance.
    DBParameterGroups: List["DBParameterGroupStatus"] = Field(default=None, frozen=True)
    #: The name of the Availability Zone where the DB instance is located.
    AvailabilityZone: Optional[str] = None
    #: Information about the subnet group associated with the DB instance, including
    #: the name, description, and subnets in the subnet group.
    DBSubnetGroup: RDSDBSubnetGroup = Field(default=None, frozen=True)
    #: The weekly time range during which system maintenance can occur, in Universal
    #: Coordinated Time (UTC).
    PreferredMaintenanceWindow: Optional[str] = None
    #: Information about pending changes to the DB instance. This information is
    #: returned only when there are pending changes. Specific changes are identified
    #: by subelements.
    PendingModifiedValues: RDSPendingModifiedValues = Field(default=None, frozen=True)
    #: The latest time to which a database in this DB instance can be restored with
    #: point-in-time restore.
    LatestRestorableTime: datetime = Field(default=None, frozen=True)
    #: The identifier of the source DB instance if this DB instance is a read replica.
    ReadReplicaSourceDBInstanceIdentifier: str = Field(default=None, frozen=True)
    #: The identifiers of the read replicas associated with this DB instance.
    ReadReplicaDBInstanceIdentifiers: List[str] = Field(default=None, frozen=True)
    #: The identifiers of Aurora DB clusters to which the RDS DB instance is
    #: replicated as a read replica. For example, when you create an Aurora read
    #: replica of an RDS for MySQL DB instance, the Aurora MySQL DB cluster for the
    #: Aurora read replica is shown. This output doesn't contain information about
    #: cross-Region Aurora read replicas.
    ReadReplicaDBClusterIdentifiers: List[str] = Field(default=None, frozen=True)
    #: The open mode of an Oracle read replica. The default is ``open-read-only``. For
    #: more information, see `Working with Oracle Read Replicas for Amazon RDS
    #: <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/oracle-read-
    #: replicas.html>`_ in the *Amazon RDS User Guide*.
    ReplicaMode: Optional[Literal["open-read-only", "mounted"]] = None
    #: The license model information for this DB instance. This setting doesn't apply
    #: to RDS Custom DB instances.
    LicenseModel: Optional[str] = None
    #: The Provisioned IOPS (I/O operations per second) value for the DB instance.
    Iops: Optional[int] = None
    #: The list of option group memberships for this DB instance.
    OptionGroupMemberships: List["OptionGroupMembership"] = Field(
        default=None, frozen=True
    )
    #: If present, specifies the name of the character set that this instance is
    #: associated with.
    CharacterSetName: Optional[str] = None
    #: The name of the NCHAR character set for the Oracle DB instance. This character
    #: set specifies the Unicode encoding for data stored in table columns of type
    #: NCHAR, NCLOB, or NVARCHAR2.
    NcharCharacterSetName: Optional[str] = None
    #: If present, specifies the name of the secondary Availability Zone for a DB
    #: instance with multi-AZ support.
    SecondaryAvailabilityZone: str = Field(default=None, frozen=True)
    #: The status of a read replica. If the DB instance isn't a read replica, the
    #: value is blank.
    StatusInfos: List["DBInstanceStatusInfo"] = Field(default=None, frozen=True)
    #: The storage type associated with the DB instance.
    StorageType: Optional[str] = None
    #: The ARN from the key store with which the instance is associated for TDE
    #: encryption.
    TdeCredentialArn: Optional[str] = None
    #: If the DB instance is a member of a DB cluster, indicates the name of the DB
    #: cluster that the DB instance is a member of.
    DBClusterIdentifier: Optional[str] = None
    #: If ``StorageEncrypted`` is enabled, the Amazon Web Services KMS key identifier
    #: for the encrypted DB instance.
    KmsKeyId: Optional[str] = None
    #: The Amazon Web Services Region-unique, immutable identifier for the DB
    #: instance. This identifier is found in Amazon Web Services CloudTrail log
    #: entries whenever the Amazon Web Services KMS key for the DB instance is
    #: accessed.
    DbiResourceId: str = Field(default=None, frozen=True)
    #: The identifier of the CA certificate for this DB instance.
    CACertificateIdentifier: Optional[str] = None
    #: The Active Directory Domain membership records associated with the DB instance.
    DomainMemberships: List["DomainMembership"] = Field(default=None, frozen=True)
    #: Indicates whether tags are copied from the DB instance to snapshots of the DB
    #: instance.
    CopyTagsToSnapshot: Optional[bool] = None
    #: The interval, in seconds, between points when Enhanced Monitoring metrics are
    #: collected for the DB instance.
    MonitoringInterval: Optional[int] = None
    #: The Amazon Resource Name (ARN) of the Amazon CloudWatch Logs log stream that
    #: receives the Enhanced Monitoring metrics data for the DB instance.
    EnhancedMonitoringResourceArn: str = Field(default=None, frozen=True)
    #: The ARN for the IAM role that permits RDS to send Enhanced Monitoring metrics
    #: to Amazon CloudWatch Logs.
    MonitoringRoleArn: Optional[str] = None
    #: The order of priority in which an Aurora Replica is promoted to the primary
    #: instance after a failure of the existing primary instance. For more
    #: information, see  `Fault Tolerance for an Aurora DB Cluster <https://docs.aws.a
    #: mazon.com/AmazonRDS/latest/AuroraUserGuide/Concepts.AuroraHighAvailability.html
    #: #Aurora.Managing.FaultTolerance>`_ in the *Amazon Aurora User Guide*.
    PromotionTier: Optional[int] = None
    #: The Amazon Resource Name (ARN) for the DB instance.
    DBInstanceArn: str = Field(default=None, frozen=True)
    #: The time zone of the DB instance. In most cases, the ``Timezone`` element is
    #: empty. ``Timezone`` content appears only for Microsoft SQL Server DB instances
    #: that were created with a time zone specified.
    Timezone: Optional[str] = None
    #: The Amazon Web Services KMS key identifier for encryption of Performance
    #: Insights data.
    PerformanceInsightsKMSKeyId: Optional[str] = None
    #: The number of days to retain Performance Insights data.
    PerformanceInsightsRetentionPeriod: Optional[int] = None
    #: The number of CPU cores and the number of threads per core for the DB instance
    #: class of the DB instance.
    ProcessorFeatures: Optional[List["ProcessorFeature"]] = None
    #: Indicates whether the DB instance has deletion protection enabled. The database
    #: can't be deleted when deletion protection is enabled. For more information, see
    #: `Deleting a DB Instance <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide
    #: /USER_DeleteInstance.html>`_.
    DeletionProtection: Optional[bool] = None
    #: The Amazon Web Services Identity and Access Management (IAM) roles associated
    #: with the DB instance.
    AssociatedRoles: List["DBInstanceRole"] = Field(default=None, frozen=True)
    #: The listener connection endpoint for SQL Server Always On.
    ListenerEndpoint: RDSEndpoint = Field(default=None, frozen=True)
    #: The upper limit in gibibytes (GiB) to which Amazon RDS can automatically scale
    #: the storage of the DB instance.
    MaxAllocatedStorage: Optional[int] = None
    #: The list of replicated automated backups associated with the DB instance.
    DBInstanceAutomatedBackupsReplications: List[
        "DBInstanceAutomatedBackupsReplication"
    ] = Field(default=None, frozen=True)
    #: The Amazon Resource Name (ARN) of the recovery point in Amazon Web Services
    #: Backup.
    AwsBackupRecoveryPointArn: Optional[str] = None
    #: The status of the database activity stream.
    ActivityStreamStatus: Literal["stopped", "starting", "started", "stopping"] = Field(
        default=None, frozen=True
    )
    #: The Amazon Web Services KMS key identifier used for encrypting messages in the
    #: database activity stream. The Amazon Web Services KMS key identifier is the key
    #: ARN, key ID, alias ARN, or alias name for the KMS key.
    ActivityStreamKmsKeyId: str = Field(default=None, frozen=True)
    #: The name of the Amazon Kinesis data stream used for the database activity
    #: stream.
    ActivityStreamKinesisStreamName: str = Field(default=None, frozen=True)
    #: The mode of the database activity stream. Database events such as a change or
    #: access generate an activity stream event. RDS for Oracle always handles these
    #: events asynchronously.
    ActivityStreamMode: Literal["sync", "async"] = Field(default=None, frozen=True)
    #: Indicates whether engine-native audit fields are included in the database
    #: activity stream.
    ActivityStreamEngineNativeAuditFieldsIncluded: bool = Field(
        default=None, frozen=True
    )
    #: The automation mode of the RDS Custom DB instance: ``full`` or ``all paused``.
    #: If ``full``, the DB instance automates monitoring and instance recovery. If
    #: ``all paused``, the instance pauses automation for the duration set by
    #: ``--resume-full-automation-mode-minutes``.
    AutomationMode: Optional[Literal["full", "all-paused"]] = None
    #: The number of minutes to pause the automation. When the time period ends, RDS
    #: Custom resumes full automation. The minimum value is 60 (default). The maximum
    #: value is 1,440.
    ResumeFullAutomationModeTime: datetime = Field(default=None, frozen=True)
    #: The instance profile associated with the underlying Amazon EC2 instance of an
    #: RDS Custom DB instance. The instance profile must meet the following
    #: requirements:
    CustomIamInstanceProfile: Optional[str] = None
    #: The location where automated backups and manual snapshots are stored: Amazon
    #: Web Services Outposts or the Amazon Web Services Region.
    BackupTarget: Optional[str] = None
    #: The network type of the DB instance.
    NetworkType: Optional[str] = None
    #: The status of the policy state of the activity stream.
    ActivityStreamPolicyStatus: Literal[
        "locked", "unlocked", "locking-policy", "unlocking-policy"
    ] = Field(default=None, frozen=True)
    #: The storage throughput for the DB instance.
    StorageThroughput: Optional[int] = None
    #: The Oracle system ID (Oracle SID) for a container database (CDB). The Oracle
    #: SID is also the name of the CDB. This setting is only valid for RDS Custom DB
    #: instances.
    DBSystemId: Optional[str] = None
    #: The secret managed by RDS in Amazon Web Services Secrets Manager for the master
    #: user password.
    MasterUserSecret: RDSMasterUserSecret = Field(default=None, frozen=True)
    #: The details of the DB instance's server certificate.
    CertificateDetails: RDSCertificateDetails = Field(default=None, frozen=True)
    #: The identifier of the source DB cluster if this DB instance is a read replica.
    ReadReplicaSourceDBClusterIdentifier: str = Field(default=None, frozen=True)
    #: The progress of the storage optimization operation as a percentage.
    PercentProgress: str = Field(default=None, frozen=True)

    @property
    def pk(self) -> Optional[str]:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`DBInstanceArn` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.DBInstanceArn

    @property
    def arn(self) -> Optional[str]:
        """
        Return the ARN of the model.   This is the value of the
        :py:attr:`DBInstanceArn` attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.DBInstanceArn

    @property
    def name(self) -> Optional[str]:
        """
        Return the name of the model.   This is the value of the
        :py:attr:`DBInstanceIdentifier` attribute.

        Returns:
            The name of the model instance.
        """
        return self.DBInstanceIdentifier


# =======================
# Request/Response Models
# =======================


class CreateDBInstanceResult(Boto3Model):
    #: Contains the details of an Amazon RDS DB instance.
    RDSDBInstance: DBInstance = Field(default=None, serialization_alias="DBInstance")


class CloudwatchLogsExportConfiguration(Boto3Model):
    """
    The log types to be enabled for export to CloudWatch Logs for a specific DB
    instance.

    A change to the ``CloudwatchLogsExportConfiguration`` parameter is always
    applied to the DB instance immediately. Therefore, the ``ApplyImmediately``
    parameter has no effect.

    This setting doesn't apply to RDS Custom DB instances.
    """

    #: The list of log types to enable.
    EnableLogTypes: Optional[List[str]] = None
    #: The list of log types to disable.
    DisableLogTypes: Optional[List[str]] = None


class ModifyDBInstanceResult(Boto3Model):
    #: Contains the details of an Amazon RDS DB instance.
    RDSDBInstance: DBInstance = Field(default=None, serialization_alias="DBInstance")


class DeleteDBInstanceResult(Boto3Model):
    #: Contains the details of an Amazon RDS DB instance.
    RDSDBInstance: DBInstance = Field(default=None, serialization_alias="DBInstance")


class DBInstanceMessage(Boto3Model):
    """
    Contains the result of a successful invocation of the
    ``DescribeDBInstances`` action.
    """

    #: An optional pagination token provided by a previous request. If this parameter
    #: is specified, the response includes only records beyond the marker, up to the
    #: value specified by ``MaxRecords`` .
    Marker: Optional[str] = None
    #: A list of ``DBInstance`` instances.
    DBInstances: Optional[List["DBInstance"]] = None
